@@ -33,6 +33,7 @@
 #include "ev-document-fonts.h"
 #include "ev-document-security.h"
 #include "ev-document-thumbnails.h"
+#include "ev-selection.h"
 
 typedef struct {
 	PdfDocument *document;
@@ -56,9 +57,6 @@ struct _PdfDocument
 	PopplerPSFile *ps_file;
 	gchar *password;
 
-	PopplerOrientation orientation;
-	gboolean orientation_set;
-
 	PopplerFontInfo *font_info;
 	PopplerFontsIter *fonts_iter;
 	int fonts_scanned_pages;
@@ -73,12 +71,16 @@ static void pdf_document_document_links_iface_init      (EvDocumentLinksIface
 static void pdf_document_document_fonts_iface_init      (EvDocumentFontsIface      *iface);
 static void pdf_document_find_iface_init                (EvDocumentFindIface       *iface);
 static void pdf_document_ps_exporter_iface_init         (EvPSExporterIface         *iface);
+static void pdf_selection_iface_init                    (EvSelectionIface          *iface);
 static void pdf_document_thumbnails_get_dimensions      (EvDocumentThumbnails      *document_thumbnails,
 							 gint                       page,
 							 gint                       size,
 							 gint                      *width,
 							 gint                      *height);
+static int  pdf_document_get_n_pages			(EvDocument                *document);
+
 static EvLink * ev_link_from_action (PopplerAction *action);
+static void pdf_document_search_free (PdfDocumentSearch   *search);
 
 
 G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
@@ -97,13 +99,53 @@ G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
 							pdf_document_find_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_PS_EXPORTER,
 							pdf_document_ps_exporter_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_SELECTION,
+							pdf_selection_iface_init);
 			 });
 
+
+static void
+set_rc_data (PdfDocument     *pdf_document,
+	     EvRenderContext *rc)
+{
+	if (rc->data == NULL) {
+		rc->data = poppler_document_get_page (pdf_document->document,
+						      rc->page);
+		rc->destroy = g_object_unref;
+	} else {
+		g_assert (rc->page == poppler_page_get_index (POPPLER_PAGE (rc->data)));
+	}
+}
+
+static void
+pdf_document_search_free (PdfDocumentSearch   *search)
+{
+        PdfDocument *pdf_document = search->document;
+	int n_pages;
+	int i;
+
+        if (search->idle != 0)
+                g_source_remove (search->idle);
+
+	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
+	for (i = 0; i < n_pages; i++) {
+		g_list_foreach (search->pages[i], (GFunc) g_free, NULL);
+		g_list_free (search->pages[i]);
+	}
+	
+        g_free (search->text);
+}
+
 static void
 pdf_document_dispose (GObject *object)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT(object);
 
+	if (pdf_document->search) {
+		pdf_document_search_free (pdf_document->search);
+		pdf_document->search = NULL;
+	}
+
 	if (pdf_document->document) {
 		g_object_unref (pdf_document->document);
 	}
@@ -176,24 +218,6 @@ pdf_document_save (EvDocument  *document,
 	return retval;
 }
 
-static PopplerOrientation
-get_document_orientation (PdfDocument *pdf_document)
-{
-	PopplerOrientation orientation;
-	PopplerPage *page;
-
-	/* Should prolly be smarter here and check more than first page */
-	page = poppler_document_get_page (pdf_document->document, 0);
-	if (page) {
-		orientation = poppler_page_get_orientation (page);
-	} else {
-		orientation = POPPLER_ORIENTATION_PORTRAIT;
-	}
-	g_object_unref (page);
-
-	return orientation;
-}
-
 static gboolean
 pdf_document_load (EvDocument   *document,
 		   const char   *uri,
@@ -219,15 +243,6 @@ pdf_document_get_n_pages (EvDocument *document)
 	return poppler_document_get_n_pages (PDF_DOCUMENT (document)->document);
 }
 
-/* FIXME This should not be necessary, poppler should rember it */
-static void
-set_page_orientation (PdfDocument *pdf_document, PopplerPage *page)
-{
-	if (pdf_document->orientation_set) {
-		poppler_page_set_orientation (page, pdf_document->orientation);
-	}
-}
-
 static void
 pdf_document_get_page_size (EvDocument   *document,
 			    int           page,
@@ -238,7 +253,6 @@ pdf_document_get_page_size (EvDocument   *document,
 	PopplerPage *poppler_page;
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 	poppler_page_get_size (poppler_page, width, height);
 	g_object_unref (poppler_page);
 }
@@ -303,36 +317,38 @@ pdf_document_get_links (EvDocument *document,
 
 static GdkPixbuf *
 pdf_document_render_pixbuf (EvDocument   *document,
-			    int           page,
-			    double        scale)
+			    EvRenderContext *rc)
 {
 	PdfDocument *pdf_document;
-	PopplerPage *poppler_page;
 	GdkPixbuf *pixbuf;
 	double width_points, height_points;
 	gint width, height;
 
 	pdf_document = PDF_DOCUMENT (document);
-	poppler_page = poppler_document_get_page (pdf_document->document,
-						  page);
-	set_page_orientation (pdf_document, poppler_page);
 
-	poppler_page_get_size (poppler_page, &width_points, &height_points);
-	width = (int) ((width_points * scale) + 0.5);
-	height = (int) ((height_points * scale) + 0.5);
+	set_rc_data (pdf_document, rc);
+
+	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
+
+	if (rc->rotation == 90 || rc->rotation == 270) {
+		width = (int) ((height_points * rc->scale) + 0.5);
+		height = (int) ((width_points * rc->scale) + 0.5);
+	} else {
+		width = (int) ((width_points * rc->scale) + 0.5);
+		height = (int) ((height_points * rc->scale) + 0.5);
+	}
 
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 				 FALSE, 8,
 				 width, height);
 
-	poppler_page_render_to_pixbuf (poppler_page,
+	poppler_page_render_to_pixbuf (POPPLER_PAGE (rc->data),
 				       0, 0,
 				       width, height,
-				       scale,
-				       pixbuf,
-				       0, 0);
+				       rc->scale,
+				       rc->rotation,
+				       pixbuf);
 	
-	g_object_unref (poppler_page);
 	
 	return pixbuf;
 }
@@ -511,7 +527,6 @@ pdf_document_get_text (EvDocument *document, int page, EvRectangle *rect)
 	char *text;
 	
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	poppler_page_get_size (poppler_page, NULL, &height);
@@ -527,59 +542,6 @@ pdf_document_get_text (EvDocument *document, int page, EvRectangle *rect)
 	return text;
 }
 
-static EvOrientation
-pdf_document_get_orientation (EvDocument *document)
-{
-	EvOrientation result;
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
-
-	if (!pdf_document->orientation_set) {
-		pdf_document->orientation = get_document_orientation (pdf_document);
-	}
-	
-	switch (pdf_document->orientation) {
-		case POPPLER_ORIENTATION_PORTRAIT:
-			result = EV_ORIENTATION_PORTRAIT;
-			break;
-		case POPPLER_ORIENTATION_LANDSCAPE:
-			result = EV_ORIENTATION_LANDSCAPE;
-			break;
-		case POPPLER_ORIENTATION_UPSIDEDOWN:
-			result = EV_ORIENTATION_UPSIDEDOWN;
-			break;
-		case POPPLER_ORIENTATION_SEASCAPE:
-			result = EV_ORIENTATION_SEASCAPE;
-			break;
-	}
-
-	return result;
-}
-
-static void
-pdf_document_set_orientation (EvDocument *document, EvOrientation orientation)
-{
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
-	PopplerOrientation poppler_orientation;
-
-	switch (orientation) {
-		case EV_ORIENTATION_PORTRAIT:
-			poppler_orientation = POPPLER_ORIENTATION_PORTRAIT;
-			break;
-		case EV_ORIENTATION_LANDSCAPE:
-			poppler_orientation = POPPLER_ORIENTATION_LANDSCAPE;
-			break;
-		case EV_ORIENTATION_UPSIDEDOWN:
-			poppler_orientation = POPPLER_ORIENTATION_UPSIDEDOWN;
-			break;
-		case EV_ORIENTATION_SEASCAPE:
-			poppler_orientation = POPPLER_ORIENTATION_SEASCAPE;
-			break;
-	}
-
-	pdf_document->orientation = poppler_orientation;
-	pdf_document->orientation_set = TRUE;
-}
-
 static void
 pdf_document_document_iface_init (EvDocumentIface *iface)
 {
@@ -593,8 +555,6 @@ pdf_document_document_iface_init (EvDocumentIface *iface)
 	iface->get_text = pdf_document_get_text;
 	iface->can_get_text = pdf_document_can_get_text;
 	iface->get_info = pdf_document_get_info;
-	iface->set_orientation = pdf_document_set_orientation;
-	iface->get_orientation = pdf_document_get_orientation;
 };
 
 static void
@@ -645,6 +605,30 @@ pdf_document_fonts_scan (EvDocumentFonts *document_fonts,
 	return result;
 }
 
+static const char *
+font_type_to_string (PopplerFontType type)
+{
+	switch (type)
+	{
+	case POPPLER_FONT_TYPE_TYPE1:
+		return _("Type 1");
+	case POPPLER_FONT_TYPE_TYPE1C:
+		return _("Type 1C");
+	case POPPLER_FONT_TYPE_TYPE3:
+		return _("Type 3");
+	case POPPLER_FONT_TYPE_TRUETYPE:
+		return _("TrueType");
+	case POPPLER_FONT_TYPE_CID_TYPE0:
+		return _("Type 1 (CID)");
+	case POPPLER_FONT_TYPE_CID_TYPE0C:
+		return _("Type 1C (CID)");
+	case POPPLER_FONT_TYPE_CID_TYPE2:
+		return _("TrueType (CID)");
+	default:
+		return _("Unknown font type");
+	}
+}
+
 static void
 pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
 			       GtkTreeModel    *model)
@@ -654,23 +638,44 @@ pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
 
 	g_return_if_fail (PDF_IS_DOCUMENT (document_fonts));
 
-	if (iter) {
-		do {
-			GtkTreeIter list_iter;
-			PopplerIndexIter *child;
-			const char *name;
-		
-			name = poppler_fonts_iter_get_name (iter);
-			if (name == NULL) {
-				name = _("No name");
-			}
+	if (!iter)
+		return;
 
-			gtk_list_store_append (GTK_LIST_STORE (model), &list_iter);
-			gtk_list_store_set (GTK_LIST_STORE (model), &list_iter,
-					    EV_DOCUMENT_FONTS_COLUMN_NAME, name,
-					    -1);
-		} while (poppler_fonts_iter_next (iter));
-	}
+	do {
+		GtkTreeIter list_iter;
+		const char *name;
+		const char *type;
+		const char *embedded;
+		char *details;
+		
+		name = poppler_fonts_iter_get_name (iter);
+
+		if (name == NULL) {
+			name = _("No name");
+		}
+
+		type = font_type_to_string (
+			poppler_fonts_iter_get_font_type (iter));
+
+		if (poppler_fonts_iter_is_embedded (iter)) {
+			if (poppler_fonts_iter_is_subset (iter))
+				embedded = _("Embedded subset");
+			else
+				embedded = _("Embedded");
+		} else {
+			embedded = _("Not embedded");
+		}
+
+		details = g_markup_printf_escaped ("%s\n%s", type, embedded);
+
+		gtk_list_store_append (GTK_LIST_STORE (model), &list_iter);
+		gtk_list_store_set (GTK_LIST_STORE (model), &list_iter,
+				    EV_DOCUMENT_FONTS_COLUMN_NAME, name,
+				    EV_DOCUMENT_FONTS_COLUMN_DETAILS, details,
+				    -1);
+
+		g_free (details);
+	} while (poppler_fonts_iter_next (iter));
 }
 
 static void
@@ -783,42 +788,56 @@ pdf_document_document_links_iface_init (EvDocumentLinksIface *iface)
 }
 
 static GdkPixbuf *
-make_thumbnail_for_size (PdfDocument *pdf_document,
-			 gint         page,
-			 gint         size,
-			 gboolean     border)
+make_thumbnail_for_size (PdfDocument   *pdf_document,
+			 gint           page,
+			 int            rotation,
+			 gint           size,
+			 gboolean       border)
 {
 	PopplerPage *poppler_page;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *sub_pixbuf;
 	int width, height;
-	int x_offset, y_offset;
 	double scale;
 	gdouble unscaled_width, unscaled_height;
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
-	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document), page, size, &width, &height);
+	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document), page,
+						size, &width, &height);
 	poppler_page_get_size (poppler_page, &unscaled_width, &unscaled_height);
 	scale = width / unscaled_width;
 
 	if (border) {
-		pixbuf = ev_document_misc_get_thumbnail_frame (width, height, NULL);
-		x_offset = 1;
-		y_offset = 1;
+		pixbuf = ev_document_misc_get_thumbnail_frame (width, height, rotation, NULL);
+
+		width = gdk_pixbuf_get_width (pixbuf);
+		height = gdk_pixbuf_get_height (pixbuf);
+		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
+						       1, 1,
+						       width - 1, height - 1);
 	} else {
+		/* rotate */
+		if (rotation == 90 || rotation == 270) {
+			int temp;
+			temp = width;
+			width = height;
+			height = temp;
+		}
+
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
 					 width, height);
 		gdk_pixbuf_fill (pixbuf, 0xffffffff);
-		x_offset = 0;
-		y_offset = 0;
+		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
+						       0, 0,
+						       width, height);
 	}
 
 	poppler_page_render_to_pixbuf (poppler_page, 0, 0,
 				       width, height,
-				       scale, pixbuf,
-				       x_offset, y_offset);
+				       scale, rotation, sub_pixbuf);
+
+	g_object_unref (G_OBJECT (sub_pixbuf));
 
 	g_object_unref (poppler_page);
 	return pixbuf;
@@ -827,6 +846,7 @@ make_thumbnail_for_size (PdfDocument *pdf_document,
 static GdkPixbuf *
 pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails,
 	 			       gint 		     page,
+				       gint                  rotation,
  				       gint                  size,
 		 		       gboolean              border)
 {
@@ -837,7 +857,6 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 	pdf_document = PDF_DOCUMENT (document_thumbnails);
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	pixbuf = poppler_page_get_thumbnail (poppler_page);
@@ -847,13 +866,13 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 		if (border) {
 			GdkPixbuf *real_pixbuf;
 
-			real_pixbuf = ev_document_misc_get_thumbnail_frame (-1, -1, pixbuf);
+			real_pixbuf = ev_document_misc_get_thumbnail_frame (-1, -1, rotation, pixbuf);
 			g_object_unref (pixbuf);
 			pixbuf = real_pixbuf;
 		}
 	} else {
 		/* There is no provided thumbnail.  We need to make one. */
-		pixbuf = make_thumbnail_for_size (pdf_document, page, size, border);
+		pixbuf = make_thumbnail_for_size (pdf_document, page, rotation, size, border);
 	}
 
 	g_object_unref (poppler_page);
@@ -874,7 +893,6 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 	
 	pdf_document = PDF_DOCUMENT (document_thumbnails);
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 
 	g_return_if_fail (width != NULL);
 	g_return_if_fail (height != NULL);
@@ -916,7 +934,6 @@ pdf_document_search_idle_callback (void *data)
 
 	page = poppler_document_get_page (search->document->document,
 					  search->search_page);
-	set_page_orientation (pdf_document, page);
 
 	ev_document_doc_mutex_lock ();
 	matches = poppler_page_find_text (page, search->text);
@@ -978,25 +995,6 @@ pdf_document_search_new (PdfDocument *pdf_document,
 	return search;
 }
 
-static void
-pdf_document_search_free (PdfDocumentSearch   *search)
-{
-        PdfDocument *pdf_document = search->document;
-	int n_pages;
-	int i;
-
-        if (search->idle != 0)
-                g_source_remove (search->idle);
-
-	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
-	for (i = 0; i < n_pages; i++) {
-		g_list_foreach (search->pages[i], (GFunc) g_free, NULL);
-		g_list_free (search->pages[i]);
-	}
-	
-        g_free (search->text);
-}
-
 static void
 pdf_document_find_begin (EvDocumentFind   *document,
 			 int               page,
@@ -1055,7 +1053,6 @@ pdf_document_find_get_result (EvDocumentFind *document_find,
 		return FALSE;
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
 	poppler_page_get_size (poppler_page, NULL, &height);
 	rectangle->x1 = r->x1;
 	rectangle->y1 = height - r->y2;
@@ -1125,25 +1122,27 @@ pdf_document_find_iface_init (EvDocumentFindIface *iface)
 
 static void
 pdf_document_ps_exporter_begin (EvPSExporter *exporter, const char *filename,
-				int first_page, int last_page)
+				int first_page, int last_page,
+				double width, double height, gboolean duplex)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	
 	pdf_document->ps_file = poppler_ps_file_new (pdf_document->document, filename,
 						     first_page,
 						     last_page - first_page + 1);
+	poppler_ps_file_set_paper_size (pdf_document->ps_file, width, height);
+	poppler_ps_file_set_duplex (pdf_document->ps_file, duplex);
 }
 
 static void
-pdf_document_ps_exporter_do_page (EvPSExporter *exporter, int page)
+pdf_document_ps_exporter_do_page (EvPSExporter *exporter, EvRenderContext *rc)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	PopplerPage *poppler_page;
 
 	g_return_if_fail (pdf_document->ps_file != NULL);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page);
+	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
 	poppler_page_render_to_ps (poppler_page, pdf_document->ps_file);
 	g_object_unref (poppler_page);
 }
@@ -1165,6 +1164,89 @@ pdf_document_ps_exporter_iface_init (EvPSExporterIface *iface)
         iface->end = pdf_document_ps_exporter_end;
 }
 
+
+void
+pdf_selection_render_selection (EvSelection      *selection,
+				EvRenderContext  *rc,
+				GdkPixbuf       **pixbuf,
+				EvRectangle      *points,
+				EvRectangle      *old_points,
+				GdkColor        *text,
+				GdkColor        *base)
+{
+	PdfDocument *pdf_document;
+	double width_points, height_points;
+	gint width, height;
+
+	pdf_document = PDF_DOCUMENT (selection);
+	set_rc_data (pdf_document, rc);
+
+	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
+	width = (int) ((width_points * rc->scale) + 0.5);
+	height = (int) ((height_points * rc->scale) + 0.5);
+
+	if (*pixbuf == NULL) {
+		* pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+					   TRUE, 8,
+					   width, height);
+	}
+	
+	poppler_page_render_selection (POPPLER_PAGE (rc->data),
+				       rc->scale, rc->rotation, *pixbuf,
+				       (PopplerRectangle *)points,
+				       (PopplerRectangle *)old_points,
+				       text,
+				       base);
+}
+
+
+GdkRegion *
+pdf_selection_get_selection_region (EvSelection     *selection,
+				    EvRenderContext *rc,
+				    EvRectangle     *points)
+{
+	PdfDocument *pdf_document;
+	GdkRegion *retval;
+
+	pdf_document = PDF_DOCUMENT (selection);
+
+	set_rc_data (pdf_document, rc);
+
+	retval = poppler_page_get_selection_region ((PopplerPage *)rc->data, rc->scale, (PopplerRectangle *) points);
+
+	return retval;
+}
+
+GdkRegion *
+pdf_selection_get_selection_map (EvSelection     *selection,
+				 EvRenderContext *rc)
+{
+	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
+	PopplerRectangle points;
+	GdkRegion *retval;
+
+	pdf_document = PDF_DOCUMENT (selection);
+	poppler_page = poppler_document_get_page (pdf_document->document,
+						  rc->page);
+
+	points.x1 = 0.0;
+	points.y1 = 0.0;
+	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
+	retval = poppler_page_get_selection_region (poppler_page, 1.0, &points);
+	g_object_unref (poppler_page);
+
+	return retval;
+}
+
+static void
+pdf_selection_iface_init (EvSelectionIface *iface)
+{
+        iface->render_selection = pdf_selection_render_selection;
+        iface->get_selection_region = pdf_selection_get_selection_region;
+        iface->get_selection_map = pdf_selection_get_selection_map;
+}
+
 PdfDocument *
 pdf_document_new (void)
 {
