@@ -60,12 +60,6 @@ gtk_gs_defaults_get_interpreter_cmd()
     return GS_PATH;
 }
 
-const gchar *
-gtk_gs_defaults_get_alpha_parameters (void)
-{
-	return ALPHA_PARAMS;
-}
-
 const gchar *
 gtk_gs_defaults_get_ungzip_cmd (void)
 {
