@@ -258,9 +258,10 @@ idle_print_handler (EvPrintJob *job)
 {
 	if (!job->printing) {
 		ev_document_doc_mutex_lock ();
-		ev_ps_exporter_begin (EV_PS_EXPORTER (job->document),
-				      job->temp_file, job->first_page,
-				      job->last_page);
+		ev_ps_exporter_begin (
+                        EV_PS_EXPORTER (job->document),
+                        job->temp_file, job->first_page, job->last_page,
+                        job->width, job->height, job->duplex);
 		ev_document_doc_mutex_unlock ();
 		job->next_page = job->first_page;
 		job->printing = TRUE;
@@ -268,13 +269,18 @@ idle_print_handler (EvPrintJob *job)
 	}
 
 	if (job->next_page <= job->last_page) {
+		EvRenderContext *rc;
 #if 0
 		g_printerr ("Printing page %d\n", job->next_page);
 #endif
+		rc = ev_render_context_new (0, job->next_page, 1.0);
+
 		ev_document_doc_mutex_lock ();
-		ev_ps_exporter_do_page (EV_PS_EXPORTER (job->document),
-					job->next_page);
+		ev_ps_exporter_do_page (EV_PS_EXPORTER (job->document), rc);
 		ev_document_doc_mutex_unlock ();
+
+		g_object_unref (rc);
+
 		job->next_page++;
 		return TRUE;
 	} else { /* no more pages */
