@@ -28,10 +28,12 @@
 G_BEGIN_DECLS
 
 #define EV_TYPE_VIEW            (ev_view_get_type ())
-#define EV_TYPE_SIZING_MODE     (ev_sizing_mode_get_type())
 #define EV_VIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), EV_TYPE_VIEW, EvView))
 #define EV_IS_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EV_TYPE_VIEW))
 
+#define EV_TYPE_SIZING_MODE     (ev_sizing_mode_get_type())
+#define EV_SIZING_MODE_CLASS    (g_type_class_peek (EV_TYPE_SIZING_MODE))
+
 typedef struct _EvView       EvView;
 typedef struct _EvViewClass  EvViewClass;
 
@@ -42,6 +44,11 @@ typedef enum {
 	EV_SIZING_FREE,
 } EvSizingMode;
 
+typedef enum {
+	EV_VIEW_SELECTION_TEXT,
+	EV_VIEW_SELECTION_RECTANGLE,
+} EvViewSelectionMode;
+
 typedef enum {
 	EV_SCROLL_PAGE_FORWARD,
 	EV_SCROLL_PAGE_BACKWARD
@@ -61,8 +68,10 @@ void		ev_view_select_all	  (EvView         *view);
 /* These are all orthoganal to each other, except 'presentation' trumps all
  * other behaviors
  */
+gboolean	ev_view_get_continuous	  (EvView         *view);
 void     	ev_view_set_continuous    (EvView         *view,
 					   gboolean        continuous);
+gboolean	ev_view_get_dual_page	  (EvView         *view);
 void     	ev_view_set_dual_page	  (EvView         *view,
 					   gboolean        dual_page);
 void     	ev_view_set_fullscreen	  (EvView         *view,
@@ -85,12 +94,17 @@ void		ev_view_set_zoom	  (EvView         *view,
 					   double          factor,
 					   gboolean        relative);
 double		ev_view_get_zoom	  (EvView         *view);
+void		ev_view_set_zoom_for_size (EvView         *view,
+					   int             width,
+		    			   int             height,
+					   int             vsb_width,
+					   int             hsb_height);
+void		ev_view_rotate_left       (EvView         *view);
+void            ev_view_rotate_right      (EvView         *view);
+void            ev_view_set_rotation      (EvView         *view,
+					   int             rotation);
+int             ev_view_get_rotation      (EvView         *view);
 
-void		ev_view_set_zoom_for_size (EvView *view,
-					   int     width,
-		    			   int     height,
-					   int     vsb_width,
-					   int     hsb_height);
 /* Find */
 gboolean	ev_view_can_find_next	  (EvView         *view);
 void            ev_view_find_next         (EvView         *view);
